@@ -36,11 +36,11 @@ import kombu.messaging
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import eventletutils
+from oslo_utils import netutils
 
 import oslo_messaging
 from oslo_messaging._drivers import amqp as rpc_amqp
 from oslo_messaging._drivers import amqpdriver
-from oslo_messaging._drivers import base
 from oslo_messaging._drivers import common as rpc_common
 from oslo_messaging._drivers import pool
 from oslo_messaging import _utils
@@ -57,13 +57,18 @@ QuorumMemConfig = collections.namedtuple('QuorumMemConfig',
 TCP_USER_TIMEOUT = 18
 
 rabbit_opts = [
+    cfg.IntOpt('rpc_conn_pool_size', default=30,
+               help='Size of RPC connection pool.',
+               min=1),
+    cfg.IntOpt('conn_pool_min_size', default=2,
+               help='The pool size limit for connections expiration policy'),
+    cfg.IntOpt('conn_pool_ttl', default=1200,
+               help='The time-to-live in sec of idle connections in the pool'),
     cfg.BoolOpt('ssl',
                 default=False,
-                deprecated_name='rabbit_use_ssl',
                 help='Connect over SSL.'),
     cfg.StrOpt('ssl_version',
                default='',
-               deprecated_name='kombu_ssl_version',
                help='SSL version to use (valid only if SSL enabled). '
                     'Valid values are TLSv1 and SSLv23. SSLv2, SSLv3, '
                     'TLSv1_1, and TLSv1_2 may be available on some '
@@ -71,19 +76,19 @@ rabbit_opts = [
                ),
     cfg.StrOpt('ssl_key_file',
                default='',
-               deprecated_name='kombu_ssl_keyfile',
                help='SSL key file (valid only if SSL enabled).'),
     cfg.StrOpt('ssl_cert_file',
                default='',
-               deprecated_name='kombu_ssl_certfile',
                help='SSL cert file (valid only if SSL enabled).'),
     cfg.StrOpt('ssl_ca_file',
                default='',
-               deprecated_name='kombu_ssl_ca_certs',
                help='SSL certification authority file '
                     '(valid only if SSL enabled).'),
     cfg.BoolOpt('ssl_enforce_fips_mode',
                 default=False,
+                deprecated_for_removal=True,
+                deprecated_reason='FIPS_mode_set API was removed in OpenSSL '
+                                  '3.0.0. This option has no effect now.',
                 help='Global toggle for enforcing the OpenSSL FIPS mode. '
                 'This feature requires Python support. '
                 'This is available in Python 3.9 in all '
@@ -114,9 +119,13 @@ rabbit_opts = [
                  default=1.0,
                  min=0.0,
                  max=amqpdriver.ACK_REQUEUE_EVERY_SECONDS_MAX * 0.9,
-                 deprecated_group='DEFAULT',
                  help='How long to wait (in seconds) before reconnecting in '
                       'response to an AMQP consumer cancel notification.'),
+    cfg.FloatOpt('kombu_reconnect_splay',
+                 default=0.0,
+                 min=0.0,
+                 help='Random time to wait for when reconnecting in response '
+                      'to an AMQP consumer cancel notification.'),
     cfg.StrOpt('kombu_compression',
                help="EXPERIMENTAL: Possible values are: gzip, bz2. If not "
                     "set compression will not be used. This option may not "
@@ -137,23 +146,22 @@ rabbit_opts = [
     cfg.StrOpt('rabbit_login_method',
                choices=('PLAIN', 'AMQPLAIN', 'EXTERNAL', 'RABBIT-CR-DEMO'),
                default='AMQPLAIN',
-               deprecated_group='DEFAULT',
                help='The RabbitMQ login method.'),
     cfg.IntOpt('rabbit_retry_interval',
+               min=1,
                default=1,
                help='How frequently to retry connecting with RabbitMQ.'),
     cfg.IntOpt('rabbit_retry_backoff',
                default=2,
-               deprecated_group='DEFAULT',
+               min=0,
                help='How long to backoff for between retries when connecting '
                     'to RabbitMQ.'),
     cfg.IntOpt('rabbit_interval_max',
                default=30,
-               help='Maximum interval of RabbitMQ connection retries. '
-                    'Default is 30 seconds.'),
+               min=1,
+               help='Maximum interval of RabbitMQ connection retries.'),
     cfg.BoolOpt('rabbit_ha_queues',
                 default=False,
-                deprecated_group='DEFAULT',
                 help='Try to use HA queues in RabbitMQ (x-ha-policy: all). '
                 'If you change this option, you must wipe the RabbitMQ '
                 'database. In RabbitMQ 3.0, queue mirroring is no longer '
@@ -190,7 +198,6 @@ rabbit_opts = [
                'Used only when rabbit_quorum_queue is enabled, '
                'Default 0 which means dont set a limit.'),
     cfg.IntOpt('rabbit_quorum_max_memory_length',
-               deprecated_name='rabbit_quroum_max_memory_length',
                default=0,
                help='By default all messages are maintained in memory '
                'if a quorum queue grows in length it can put memory '
@@ -199,7 +206,6 @@ rabbit_opts = [
                'Used only when rabbit_quorum_queue is enabled, '
                'Default 0 which means dont set a limit.'),
     cfg.IntOpt('rabbit_quorum_max_memory_bytes',
-               deprecated_name='rabbit_quroum_max_memory_bytes',
                default=0,
                help='By default all messages are maintained in memory '
                'if a quorum queue grows in length it can put memory '
@@ -217,7 +223,14 @@ rabbit_opts = [
                     '0 as value will disable the x-expires. If doing so, '
                     'make sure you have a rabbitmq policy to delete the '
                     'queues or you deployment will create an infinite number '
-                    'of queue over time.'),
+                    'of queue over time.'
+                    'In case rabbit_stream_fanout is set to True, this option '
+                    'will control data retention policy (x-max-age) for '
+                    'messages in the fanout queue rather then the queue '
+                    'duration itself. So the oldest data in the stream queue '
+                    'will be discarded from it once reaching TTL '
+                    'Setting to 0 will disable x-max-age for stream which '
+                    'make stream grow indefinitely filling up the diskspace'),
     cfg.IntOpt('rabbit_qos_prefetch_count',
                default=0,
                help='Specifies the number of messages to prefetch. Setting to '
@@ -257,6 +270,7 @@ rabbit_opts = [
                help='Hostname used by queue manager. Defaults to the value '
                'returned by socket.gethostname().'),
     cfg.StrOpt('processname',
+               sample_default='nova-api',
                default=os.path.basename(sys.argv[0]),
                help='Process name used by queue manager'),
     cfg.BoolOpt('rabbit_stream_fanout',
@@ -358,7 +372,7 @@ def _get_queue_arguments(rabbit_ha_queues, rabbit_queue_ttl,
 
 class RabbitMessage(dict):
     def __init__(self, raw_message):
-        super(RabbitMessage, self).__init__(
+        super().__init__(
             rpc_common.deserialize_msg(raw_message.payload))
         LOG.trace('RabbitMessage.Init: message %s', self)
         self._raw_message = raw_message
@@ -372,7 +386,7 @@ class RabbitMessage(dict):
         self._raw_message.requeue()
 
 
-class Consumer(object):
+class Consumer:
     """Consumer class."""
 
     def __init__(self, exchange_name, queue_name, routing_key, type, durable,
@@ -393,7 +407,6 @@ class Consumer(object):
         self.callback = callback
         self.type = type
         self.nowait = nowait
-        rabbit_quorum_queue_config = rabbit_quorum_queue_config or {}
         self.queue_arguments = _get_queue_arguments(
             rabbit_ha_queues, rabbit_queue_ttl, rabbit_quorum_queue,
             rabbit_quorum_queue_config, rabbit_stream_fanout)
@@ -408,7 +421,7 @@ class Consumer(object):
         self.rabbit_stream_fanout = rabbit_stream_fanout
         self.next_stream_offset = "last"
 
-    def _declare_fallback(self, err, conn, consumer_arguments):
+    def _declare_fallback_nondurable(self, err, conn, consumer_arguments):
         """Fallback by declaring a non durable queue.
 
         When a control exchange is shared between services it is possible
@@ -420,32 +433,31 @@ class Consumer(object):
         to fallback by creating a non durable queue to match the default
         config.
         """
-        if "PRECONDITION_FAILED - inequivalent arg 'durable'" in str(err):
-            LOG.info(
-                "[%s] Retrying to declare the exchange (%s) as "
-                "non durable", conn.connection_id, self.exchange_name)
-            self.exchange = kombu.entity.Exchange(
-                name=self.exchange_name,
-                type=self.type,
-                durable=False,
-                auto_delete=self.queue_auto_delete)
-            self.queue = kombu.entity.Queue(
-                name=self.queue_name,
-                channel=conn.channel,
-                exchange=self.exchange,
-                durable=False,
-                auto_delete=self.queue_auto_delete,
-                routing_key=self.routing_key,
-                queue_arguments=self.queue_arguments,
-                consumer_arguments=consumer_arguments
-            )
-            self.queue.declare()
+        LOG.info(
+            "[%s] Retrying to declare the exchange (%s) as "
+            "non durable", conn.connection_id, self.exchange_name)
+        self.exchange = kombu.entity.Exchange(
+            name=self.exchange_name,
+            type=self.type,
+            durable=False,
+            auto_delete=self.queue_auto_delete)
+        self.queue = kombu.entity.Queue(
+            name=self.queue_name,
+            channel=conn.channel,
+            exchange=self.exchange,
+            durable=False,
+            auto_delete=self.queue_auto_delete,
+            routing_key=self.routing_key,
+            queue_arguments=self.queue_arguments,
+            consumer_arguments=consumer_arguments
+        )
+        self.queue.declare()
 
     def reset_stream_offset(self):
         if not self.rabbit_stream_fanout:
             return
-        LOG.warn("Reset consumer for queue %s next offset was at %s.",
-                 self.queue_name, self.next_stream_offset)
+        LOG.warning("Reset consumer for queue %s next offset was at %s.",
+                    self.queue_name, self.next_stream_offset)
         self.next_stream_offset = "last"
 
     def declare(self, conn):
@@ -482,16 +494,21 @@ class Consumer(object):
             try:
                 self.queue.declare()
             except amqp_ex.PreconditionFailed as err:
-                # NOTE(hberaud): This kind of exception may be triggered
-                # when a control exchange is shared between services and
-                # when services try to create it with configs that differ
-                # from each others. RabbitMQ will reject the services
-                # that try to create it with a configuration that differ
-                # from the one used first.
-                LOG.warning('[%s] Queue %s could not be declared probably '
-                            'because of conflicting configurations: %s',
-                            conn.connection_id, self.queue_name, err)
-                self._declare_fallback(err, conn, consumer_arguments)
+                if "PRECONDITION_FAILED - inequivalent arg 'durable'" in \
+                        str(err):
+                    # NOTE(hberaud): This kind of exception may be triggered
+                    # when a control exchange is shared between services and
+                    # when services try to create it with configs that differ
+                    # from each others. RabbitMQ will reject the services
+                    # that try to create it with a configuration that differ
+                    # from the one used first.
+                    LOG.warning('[%s] Queue %s could not be declared probably '
+                                'because of conflicting configurations: %s',
+                                conn.connection_id, self.queue_name, err)
+                    self._declare_fallback_nondurable(
+                        err, conn, consumer_arguments)
+                else:
+                    raise
             except amqp_ex.NotFound as ex:
                 # NOTE(viktor.krivak): This exception is raised when
                 # non-durable and non-ha queue is hosted on node that
@@ -544,14 +561,6 @@ class Consumer(object):
                           'Queue: [%(queue)s], '
                           'error message: [%(err_str)s]', info)
                 time.sleep(interval)
-                if self.queue_arguments.get('x-queue-type') == 'quorum':
-                    # Before re-declare queue, try to delete it
-                    # This is helping with issue #2028384
-                    # NOTE(amorin) we need to make sure the connection is
-                    # established again, because when an error occur, the
-                    # connection is closed.
-                    conn.ensure_connection()
-                    self.queue.delete()
                 self.queue.declare()
             else:
                 raise
@@ -594,24 +603,6 @@ class Consumer(object):
                                    nowait=self.nowait)
             else:
                 raise
-        except amqp_ex.InternalError as exc:
-            if self.queue_arguments.get('x-queue-type') == 'quorum':
-                # Before re-consume queue, try to delete it
-                # This is helping with issue #2028384
-                if exc.code == 541:
-                    LOG.warning('Queue %s seems broken, will try delete it '
-                                'before starting over.', self.queue.name)
-                    # NOTE(amorin) we need to make sure the connection is
-                    # established again, because when an error occur, the
-                    # connection is closed.
-                    conn.ensure_connection()
-                    self.queue.delete()
-                    self.declare(conn)
-                    self.queue.consume(callback=self._callback,
-                                       consumer_tag=str(tag),
-                                       nowait=self.nowait)
-            else:
-                raise
 
     def cancel(self, tag):
         LOG.trace('ConsumerBase.cancel: canceling %s', tag)
@@ -715,7 +706,7 @@ class ConnectionLock(DummyConnectionLock):
             self.release()
 
 
-class Connection(object):
+class Connection:
     """Connection object."""
 
     def __init__(self, conf, url, purpose, retry=None):
@@ -742,6 +733,7 @@ class Connection(object):
             driver_conf.heartbeat_timeout_threshold
         self.heartbeat_rate = driver_conf.heartbeat_rate
         self.kombu_reconnect_delay = driver_conf.kombu_reconnect_delay
+        self.kombu_reconnect_splay = driver_conf.kombu_reconnect_splay
         self.amqp_durable_queues = driver_conf.amqp_durable_queues
         self.amqp_auto_delete = driver_conf.amqp_auto_delete
         self.ssl = driver_conf.ssl
@@ -802,19 +794,6 @@ class Connection(object):
             self.ssl_cert_file = driver_conf.ssl_cert_file
             self.ssl_ca_file = driver_conf.ssl_ca_file
 
-            if self.ssl_enforce_fips_mode:
-                if hasattr(ssl, 'FIPS_mode'):
-                    LOG.info("Enforcing the use of the OpenSSL FIPS mode")
-                    ssl.FIPS_mode_set(1)
-                else:
-                    raise exceptions.ConfigurationError(
-                        "OpenSSL FIPS mode is not supported by your Python "
-                        "version. You must either change the Python "
-                        "executable used to a version with FIPS mode "
-                        "support or disable FIPS mode by setting the "
-                        "'[oslo_messaging_rabbit] ssl_enforce_fips_mode' "
-                        "configuration option to 'False'.")
-
         self._url = ''
         if url.hosts:
             if url.transport.startswith('kombu+'):
@@ -997,20 +976,14 @@ class Connection(object):
                                  default_password='', default_hostname=''):
         transport = url.transport.replace('kombu+', '')
         transport = transport.replace('rabbit', 'amqp')
-        return '%s://%s:%s@%s:%s/%s' % (
+        return '{}://{}:{}@{}:{}/{}'.format(
             transport,
             parse.quote(host.username or default_username),
             parse.quote(host.password or default_password),
-            self._parse_url_hostname(host.hostname) or default_hostname,
+            netutils.escape_ipv6(host.hostname) or default_hostname,
             str(host.port or 5672),
             url.virtual_host or '')
 
-    def _parse_url_hostname(self, hostname):
-        """Handles hostname returned from urlparse and checks whether it's
-        ipaddress. If it's ipaddress it ensures that it has brackets for IPv6.
-        """
-        return '[%s]' % hostname if ':' in hostname else hostname
-
     def _fetch_ssl_params(self):
         """Handles fetching what ssl params should be used for the connection
         (if any).
@@ -1018,7 +991,7 @@ class Connection(object):
         if self.ssl:
             ssl_params = dict()
 
-            # http://docs.python.org/library/ssl.html - ssl.wrap_socket
+            # http://docs.python.org/library/ssl.html
             if self.ssl_version:
                 ssl_params['ssl_version'] = self.validate_ssl_version(
                     self.ssl_version)
@@ -1053,7 +1026,7 @@ class Connection(object):
         self.connection.ensure_connection(
             errback=on_error,
             max_retries=self.max_retries,
-            interval_start=self.interval_start or 1,
+            interval_start=self.interval_start,
             interval_step=self.interval_stepping,
             interval_max=self.interval_max,
         )
@@ -1083,8 +1056,8 @@ class Connection(object):
             retry = float('inf')
 
         def on_error(exc, interval):
-            LOG.debug("[%s] Received recoverable error from kombu:"
-                      % self.connection_id,
+            LOG.debug("[%s] Received recoverable error from kombu:",
+                      self.connection_id,
                       exc_info=True)
 
             recoverable_error_callback and recoverable_error_callback(exc)
@@ -1092,6 +1065,10 @@ class Connection(object):
             interval = (self.kombu_reconnect_delay + interval
                         if self.kombu_reconnect_delay > 0
                         else interval)
+            if self.kombu_reconnect_splay > 0:
+                interval += random.uniform(
+                    0,
+                    self.kombu_reconnect_splay)  # nosec
 
             info = {'err_str': exc, 'sleep_time': interval}
             info.update(self._get_connection_info(conn_error=True))
@@ -1102,9 +1079,9 @@ class Connection(object):
                 # happen, for example, when we delete the stream queue.
                 # We need to start consuming from "last" because the stream
                 # offset maybe reset.
-                LOG.warn('[%s] Basic.cancel received. '
-                         'Resetting consumers offsets to last.',
-                         self.connection_id)
+                LOG.warning('[%s] Basic.cancel received. '
+                            'Resetting consumers offsets to last.',
+                            self.connection_id)
                 for consumer in self._consumers:
                     consumer.reset_stream_offset()
 
@@ -1156,7 +1133,7 @@ class Connection(object):
                 execute_method, channel=self.channel,
                 max_retries=retry,
                 errback=on_error,
-                interval_start=self.interval_start or 1,
+                interval_start=self.interval_start,
                 interval_step=self.interval_stepping,
                 interval_max=self.interval_max,
                 on_revive=on_reconnection)
@@ -1235,7 +1212,7 @@ class Connection(object):
                 for consumer in filter(lambda c: c.type == 'fanout',
                                        self._consumers):
                     LOG.debug('[connection close] Deleting fanout '
-                              'queue: %s ' % consumer.queue.name)
+                              'queue: %s ', consumer.queue.name)
                     consumer.queue.delete()
             self._set_current_channel(None)
             self.connection.release()
@@ -1290,8 +1267,8 @@ class Connection(object):
             sock = self.channel.connection.sock
         except AttributeError as e:
             # Level is set to debug because otherwise we would spam the logs
-            LOG.debug('[%s] Failed to get socket attribute: %s'
-                      % (self.connection_id, str(e)))
+            LOG.debug('[%s] Failed to get socket attribute: %s',
+                      self.connection_id, e)
         else:
             sock.settimeout(timeout)
             # TCP_USER_TIMEOUT is not defined on Windows and Mac OS X
@@ -1306,7 +1283,7 @@ class Connection(object):
                     sock.setsockopt(socket.IPPROTO_TCP,
                                     TCP_USER_TIMEOUT,
                                     int(math.ceil(timeout)))
-                except socket.error as error:
+                except OSError as error:
                     code = error[0]
                     # TCP_USER_TIMEOUT not defined on kernels <2.6.37
                     if code != errno.ENOPROTOOPT:
@@ -1357,7 +1334,7 @@ class Connection(object):
                         # already do that for other connection
                         try:
                             self.connection.drain_events(timeout=0.001)
-                        except socket.timeout:
+                        except TimeoutError:
                             pass
                     # NOTE(hberaud): In a clustered rabbitmq when
                     # a node disappears, we get a ConnectionRefusedError
@@ -1367,9 +1344,7 @@ class Connection(object):
                     # Catch these exceptions to ensure that we call
                     # ensure_connection for switching the
                     # connection destination.
-                    except (socket.timeout,
-                            ConnectionRefusedError,
-                            OSError,
+                    except (TimeoutError, ConnectionRefusedError, OSError,
                             kombu.exceptions.OperationalError,
                             amqp_ex.ConnectionForced) as exc:
                         LOG.info("A recoverable connection/channel error "
@@ -1453,7 +1428,7 @@ class Connection(object):
                 try:
                     self.connection.drain_events(timeout=poll_timeout)
                     return
-                except socket.timeout:
+                except TimeoutError:
                     poll_timeout = timer.check_return(
                         _raise_timeout, maximum=self._poll_timeout)
                 except self.connection.channel_errors as exc:
@@ -1524,7 +1499,7 @@ class Connection(object):
                 unique = self._q_manager.get()
             else:
                 unique = uuid.uuid4().hex
-            queue_name = '%s_fanout_%s' % (topic, unique)
+            queue_name = f'{topic}_fanout_{unique}'
         LOG.debug('Creating fanout queue: %s', queue_name)
 
         is_durable = (self.rabbit_transient_quorum_queue or
@@ -1570,8 +1545,8 @@ class Connection(object):
         # the connection's socket while it is in an error state will cause
         # py-amqp to attempt reconnecting.
         ci = self.connection.info()
-        info = dict([(k, ci.get(k)) for k in
-                     ['hostname', 'port', 'transport']])
+        info = {k: ci.get(k) for k in
+                ['hostname', 'port', 'transport']}
         client_port = None
         if (not conn_error and self.channel and
                 hasattr(self.channel.connection, 'sock') and
@@ -1589,17 +1564,19 @@ class Connection(object):
             try:
                 exchange(self.channel).declare()
             except amqp_ex.PreconditionFailed as err:
-                # NOTE(hberaud): This kind of exception may be triggered
-                # when a control exchange is shared between services and
-                # when services try to create it with configs that differ
-                # from each others. RabbitMQ will reject the services
-                # that try to create it with a configuration that differ
-                # from the one used first.
                 if "PRECONDITION_FAILED - inequivalent arg 'durable'" \
                    in str(err):
+                    # NOTE(hberaud): This kind of exception may be triggered
+                    # when a control exchange is shared between services and
+                    # when services try to create it with configs that differ
+                    # from each others. RabbitMQ will reject the services
+                    # that try to create it with a configuration that differ
+                    # from the one used first.
                     LOG.warning("Force creating a non durable exchange.")
                     exchange.durable = False
                     exchange(self.channel).declare()
+                else:
+                    raise
             self._declared_exchanges.add(exchange.name)
 
         log_info = {'msg': msg,
@@ -1763,7 +1740,6 @@ class RabbitDriver(amqpdriver.AMQPDriverBase):
         conf.register_group(opt_group)
         conf.register_opts(rabbit_opts, group=opt_group)
         conf.register_opts(rpc_amqp.amqp_opts, group=opt_group)
-        conf.register_opts(base.base_opts, group=opt_group)
         conf = rpc_common.ConfigOptsProxy(conf, url, opt_group.name)
 
         self.missing_destination_retry_timeout = (
@@ -1785,12 +1761,25 @@ class RabbitDriver(amqpdriver.AMQPDriverBase):
             conf, max_size, min_size, ttl,
             url, Connection)
 
-        super(RabbitDriver, self).__init__(
+        if conf.oslo_messaging_rabbit.use_queue_manager:
+            self._q_manager = amqpdriver.QManager(
+                hostname=conf.oslo_messaging_rabbit.hostname,
+                processname=conf.oslo_messaging_rabbit.processname)
+        else:
+            self._q_manager = None
+
+        super().__init__(
             conf, url,
             connection_pool,
             default_exchange,
             allowed_remote_exmods
         )
 
+    def _get_reply_queue_name(self):
+        if self._q_manager:
+            return 'reply_' + self._q_manager.get()
+        else:
+            return super()._get_reply_queue_name()
+
     def require_features(self, requeue=True):
         pass
