@@ -16,6 +16,28 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from abc import ABCMeta
+from abc import abstractmethod
+import logging
+import sys
+import threading
+
+from oslo_config import cfg
+from oslo_utils import eventletutils
+
+from oslo_messaging import _utils as utils
+from oslo_messaging import dispatcher
+from oslo_messaging import serializer as msg_serializer
+from oslo_messaging import server as msg_server
+from oslo_messaging import target as msg_target
+
+_dispatcher_opts = [
+    cfg.BoolOpt('rpc_ping_enabled',
+                default=False,
+                help='Add an endpoint to answer to ping calls. '
+                     'Endpoint is named oslo_rpc_server_ping'),
+]
+
 __all__ = [
     'NoSuchMethod',
     'RPCAccessPolicyBase',
@@ -28,23 +50,14 @@ __all__ = [
     'ExpectedException',
 ]
 
-from abc import ABCMeta
-from abc import abstractmethod
-import logging
-import sys
-import threading
-
-import six
-
-from oslo_messaging import _utils as utils
-from oslo_messaging import dispatcher
-from oslo_messaging import serializer as msg_serializer
-from oslo_messaging import server as msg_server
-from oslo_messaging import target as msg_target
-
 LOG = logging.getLogger(__name__)
 
 
+class PingEndpoint:
+    def oslo_rpc_server_ping(self, ctxt, **kwargs):
+        return 'pong'
+
+
 class ExpectedException(Exception):
     """Encapsulates an expected exception raised by an RPC endpoint
 
@@ -65,7 +78,7 @@ class NoSuchMethod(RPCDispatcherError, AttributeError):
 
     def __init__(self, method):
         msg = "Endpoint does not support RPC method %s" % method
-        super(NoSuchMethod, self).__init__(msg)
+        super().__init__(msg)
         self.method = method
 
 
@@ -75,14 +88,13 @@ class UnsupportedVersion(RPCDispatcherError):
     def __init__(self, version, method=None):
         msg = "Endpoint does not support RPC version %s" % version
         if method:
-            msg = "%s. Attempted method: %s" % (msg, method)
-        super(UnsupportedVersion, self).__init__(msg)
+            msg = f"{msg}. Attempted method: {method}"
+        super().__init__(msg)
         self.version = version
         self.method = method
 
 
-@six.add_metaclass(ABCMeta)
-class RPCAccessPolicyBase(object):
+class RPCAccessPolicyBase(metaclass=ABCMeta):
     """Determines which endpoint methods may be invoked via RPC"""
 
     @abstractmethod
@@ -154,17 +166,40 @@ class RPCDispatcher(dispatcher.DispatcherBase):
         :param endpoints: list of endpoint objects for dispatching to
         :param serializer: optional message serializer
         """
+        cfg.CONF.register_opts(_dispatcher_opts)
+        oslo_rpc_server_ping = None
 
         for ep in endpoints:
+            # Check if we have an attribute named 'target'
             target = getattr(ep, 'target', None)
             if target and not isinstance(target, msg_target.Target):
                 errmsg = "'target' is a reserved Endpoint attribute used" + \
                          " for namespace and version filtering.  It must" + \
                          " be of type oslo_messaging.Target. Do not" + \
                          " define an Endpoint method named 'target'"
-                raise TypeError("%s: endpoint=%s" % (errmsg, ep))
+                raise TypeError(f"{errmsg}: endpoint={ep}")
+
+            # Check if we have an attribute named 'oslo_rpc_server_ping'
+            oslo_rpc_server_ping = getattr(ep, 'oslo_rpc_server_ping', None)
+            if oslo_rpc_server_ping:
+                errmsg = "'oslo_rpc_server_ping' is a reserved Endpoint" + \
+                         " attribute which can be use to ping the" + \
+                         " endpoint. Please avoid using any oslo_* " + \
+                         " naming."
+                LOG.warning(f"{errmsg} (endpoint={ep})")
 
         self.endpoints = endpoints
+
+        # Add ping endpoint if enabled in config
+        if cfg.CONF.rpc_ping_enabled:
+            if oslo_rpc_server_ping:
+                LOG.warning("rpc_ping_enabled=True in config but "
+                            "oslo_rpc_server_ping is already declared "
+                            "in an other Endpoint. Not enabling rpc_ping "
+                            "Endpoint.")
+            else:
+                self.endpoints.append(PingEndpoint())
+
         self.serializer = serializer or msg_serializer.NoOpSerializer()
         self._default_target = msg_target.Target()
         if access_policy is not None:
@@ -208,17 +243,26 @@ class RPCDispatcher(dispatcher.DispatcherBase):
             client_timeout = cm_heartbeat_interval = 0
 
         if cm_heartbeat_interval < 1:
-            LOG.warning('Client provided an invalid timeout value of %r' % (
-                incoming.client_timeout))
+            LOG.warning('Client provided an invalid timeout value of %r',
+                        incoming.client_timeout)
             return
 
         while not event.wait(cm_heartbeat_interval):
             LOG.debug(
                 'Sending call-monitor heartbeat for active call to %(method)s '
-                '(interval=%(interval)i)' % (
-                    {'method': incoming.message.get('method'),
-                     'interval': cm_heartbeat_interval}))
-            incoming.heartbeat()
+                '(interval=%(interval)i)',
+                {'method': incoming.message.get('method'),
+                 'interval': cm_heartbeat_interval})
+            try:
+                incoming.heartbeat()
+            except Exception as exc:
+                # The heartbeat message failed to send.  Likely the broker or
+                # client has died. Nothing to do here but exit the watchdog
+                # thread. If the client is still alive (dead broker) then its
+                # RPC will timeout as expected.
+                LOG.debug("Call-monitor heartbeat failed: %(exc)s",
+                          {'exc': exc})
+                break
 
     def dispatch(self, incoming):
         """Dispatch an RPC message to the appropriate endpoint method.
@@ -240,7 +284,7 @@ class RPCDispatcher(dispatcher.DispatcherBase):
         # is executing if it runs for some time. The thread will wait
         # for the event to be signaled, which we do explicitly below
         # after dispatching the method call.
-        completion_event = threading.Event()
+        completion_event = eventletutils.Event()
         watchdog_thread = threading.Thread(target=self._watchdog,
                                            args=(completion_event, incoming))
         if incoming.client_timeout:
