@@ -19,6 +19,7 @@ import threading
 import time
 import uuid
 
+from amqp import exceptions as amqp_ex
 import fixtures
 import kombu
 import kombu.connection
@@ -31,7 +32,6 @@ import oslo_messaging
 from oslo_messaging._drivers import amqpdriver
 from oslo_messaging._drivers import common as driver_common
 from oslo_messaging._drivers import impl_rabbit as rabbit_driver
-from oslo_messaging.exceptions import ConfigurationError
 from oslo_messaging.exceptions import MessageDeliveryFailure
 from oslo_messaging.tests import utils as test_utils
 from oslo_messaging.transport import DriverLoadFailure
@@ -206,66 +206,24 @@ class TestRabbitDriverLoadSSL(test_utils.BaseTestCase):
         )
 
 
-class TestRabbitDriverLoadSSLWithFIPS(test_utils.BaseTestCase):
-    scenarios = [
-        ('ssl_fips_mode', dict(options=dict(ssl=True,
-                                            ssl_enforce_fips_mode=True),
-                               expected=True)),
-    ]
-
-    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection'
-                '.ensure_connection')
-    @mock.patch('kombu.connection.Connection')
-    def test_driver_load_with_fips_supported(self,
-                                             connection_klass, fake_ensure):
-        self.config(ssl=True, ssl_enforce_fips_mode=True,
-                    group="oslo_messaging_rabbit")
-        transport = oslo_messaging.get_transport(self.conf,
-                                                 'kombu+memory:////')
-        self.addCleanup(transport.cleanup)
-
-        with mock.patch.object(ssl, 'FIPS_mode',
-                               create=True, return_value=True):
-            with mock.patch.object(ssl, 'FIPS_mode_set', create=True):
-
-                connection = transport._driver._get_connection()
-                connection_klass.assert_called_once_with(
-                    'memory:///', transport_options={
-                        'client_properties': {
-                            'capabilities': {
-                                'connection.blocked': True,
-                                'consumer_cancel_notify': True,
-                                'authentication_failure_close': True,
-                            },
-                            'connection_name': connection.name},
-                        'confirm_publish': True,
-                        'on_blocked': mock.ANY,
-                        'on_unblocked': mock.ANY},
-                    ssl=self.expected, login_method='AMQPLAIN',
-                    heartbeat=60, failover_strategy='round-robin'
-                )
-
-    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection'
-                '.ensure_connection')
-    @mock.patch('oslo_messaging._drivers.impl_rabbit.ssl')
-    @mock.patch('kombu.connection.Connection')
-    def test_fips_unsupported(self, connection_klass, fake_ssl, fake_ensure):
-        self.config(ssl=True, ssl_enforce_fips_mode=True,
-                    group="oslo_messaging_rabbit")
-        transport = oslo_messaging.get_transport(self.conf,
-                                                 'kombu+memory:////')
-        self.addCleanup(transport.cleanup)
-
-        del fake_ssl.FIPS_mode
-
-        # We do this test only if FIPS mode is not supported to
-        # ensure that we hard fail.
-        self.assertRaises(
-            ConfigurationError,
-            transport._driver._get_connection)
-
-
 class TestRabbitPublisher(test_utils.BaseTestCase):
+    @mock.patch('kombu.messaging.Producer.publish')
+    def test_send(self, fake_publish):
+        transport = oslo_messaging.get_transport(self.conf,
+                                                 'kombu+memory:////')
+        exchange_mock = mock.Mock()
+        with transport._driver._get_connection(
+                driver_common.PURPOSE_SEND) as pool_conn:
+            conn = pool_conn.connection
+            conn._publish(exchange_mock, 'msg', routing_key='routing_key')
+        exchange_mock.return_value.declare.assert_not_called()
+        fake_publish.assert_called_with(
+            'msg', expiration=None,
+            mandatory=False,
+            compression=self.conf.oslo_messaging_rabbit.kombu_compression,
+            exchange=exchange_mock,
+            routing_key='routing_key')
+
     @mock.patch('kombu.messaging.Producer.publish')
     def test_send_with_timeout(self, fake_publish):
         transport = oslo_messaging.get_transport(self.conf,
@@ -285,14 +243,53 @@ class TestRabbitPublisher(test_utils.BaseTestCase):
             routing_key='routing_key')
 
     @mock.patch('kombu.messaging.Producer.publish')
-    def test_send_no_timeout(self, fake_publish):
+    def test_send_declare(self, fake_publish):
         transport = oslo_messaging.get_transport(self.conf,
                                                  'kombu+memory:////')
         exchange_mock = mock.Mock()
+        exchange_mock.passive = False
         with transport._driver._get_connection(
                 driver_common.PURPOSE_SEND) as pool_conn:
             conn = pool_conn.connection
             conn._publish(exchange_mock, 'msg', routing_key='routing_key')
+        exchange_mock.return_value.declare.assert_called_once_with()
+        fake_publish.assert_called_with(
+            'msg', expiration=None,
+            mandatory=False,
+            compression=self.conf.oslo_messaging_rabbit.kombu_compression,
+            exchange=exchange_mock,
+            routing_key='routing_key')
+
+    @mock.patch('kombu.messaging.Producer.publish')
+    def test_send_declare_precondition_failed(self, fake_publish):
+        transport = oslo_messaging.get_transport(self.conf,
+                                                 'kombu+memory:////')
+        exchange_mock = mock.Mock()
+        exchange_mock.passive = False
+        exchange_mock.return_value.declare.side_effect = \
+            amqp_ex.PreconditionFailed('error')
+        with transport._driver._get_connection(
+                driver_common.PURPOSE_SEND) as pool_conn:
+            conn = pool_conn.connection
+            self.assertRaises(
+                amqp_ex.PreconditionFailed,
+                conn._publish, exchange_mock, 'msg', routing_key='routing_key')
+        fake_publish.assert_not_called()
+
+    @mock.patch('kombu.messaging.Producer.publish')
+    def test_send_declare_precondition_failed_durable(self, fake_publish):
+        transport = oslo_messaging.get_transport(self.conf,
+                                                 'kombu+memory:////')
+        exchange_mock = mock.Mock()
+        exchange_mock.passive = False
+        exchange_mock.return_value.declare.side_effect = [
+            amqp_ex.PreconditionFailed(
+                "PRECONDITION_FAILED - inequivalent arg 'durable'"), None]
+        with transport._driver._get_connection(
+                driver_common.PURPOSE_SEND) as pool_conn:
+            conn = pool_conn.connection
+            conn._publish(exchange_mock, 'msg', routing_key='routing_key')
+        self.assertEqual(2, exchange_mock.return_value.declare.call_count)
         fake_publish.assert_called_with(
             'msg', expiration=None,
             mandatory=False,
@@ -778,7 +775,7 @@ def _declare_queue(target):
                                          type='topic',
                                          durable=False,
                                          auto_delete=False)
-        topic = '{}.{}'.format(target.topic, target.server)
+        topic = f'{target.topic}.{target.server}'
         queue = kombu.entity.Queue(name=topic,
                                    channel=channel,
                                    exchange=exchange,
@@ -928,7 +925,7 @@ def _create_producer(target):
                                          type='topic',
                                          durable=False,
                                          auto_delete=False)
-        topic = '{}.{}'.format(target.topic, target.server)
+        topic = f'{target.topic}.{target.server}'
         producer = kombu.messaging.Producer(exchange=exchange,
                                             channel=channel,
                                             routing_key=topic)
@@ -1048,19 +1045,15 @@ class RpcKombuHATestCase(test_utils.BaseTestCase):
             'kombu.connection.Connection.connection'))
         self.useFixture(fixtures.MockPatch(
             'kombu.connection.Connection.channel'))
-        # TODO(stephenfin): Drop hasattr when we drop support for kombo < 4.6.8
-        if hasattr(kombu.connection.Connection, '_connection_factory'):
-            self.useFixture(fixtures.MockPatch(
-                'kombu.connection.Connection._connection_factory'))
+        self.useFixture(fixtures.MockPatch(
+            'kombu.connection.Connection._connection_factory'))
 
         # starting from the first broker in the list
         url = oslo_messaging.TransportURL.parse(self.conf, None)
         self.connection = rabbit_driver.Connection(self.conf, url,
                                                    driver_common.PURPOSE_SEND)
-        # TODO(stephenfin): Remove when we drop support for kombo < 4.6.8
-        if hasattr(kombu.connection.Connection, 'connect'):
-            self.useFixture(fixtures.MockPatch(
-                'kombu.connection.Connection.connect'))
+        self.useFixture(fixtures.MockPatch(
+            'kombu.connection.Connection.connect'))
         self.addCleanup(self.connection.close)
 
     def test_ensure_four_retry(self):
